//Include
#include <avr/io.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "lcd-routines.h"
#include "rtc.h"

typedef struct sMenuChild{
  const char*   text;
  struct sMenu* subMenu;
  void (* myFunction)(void);
} MenuChild;

typedef struct sMenu{
  uint8_t     nrChilds;
  MenuChild*  childs;
} Menu;

#define NR_ENTRIES(x)  (sizeof(x)/sizeof(*x))

void KommunikationAktivieren(void);
void KommunikationGeschwindigkeitEinstellen(void);
void KommunikationAdresseEinstellen(void);
void DatumUhrzeitStellen(void);

void BeleuchtungTypeEinstellen(void);

void RaumtemperaturEinstellen(void);
void AbsenktemperaturEinstellen(void);
void TemperaturTypeEinstellen(void);

uint8_t KeyPress(void);

MenuChild Kommunikation[] = {
	{"Aktiviert", NULL, KommunikationAktivieren},
	{"Geschwindigkeit", NULL, KommunikationGeschwindigkeitEinstellen},
	{"Adresse", NULL, KommunikationAdresseEinstellen},
};

Menu MenuEinstellungenKommunikation = { NR_ENTRIES( Kommunikation ), Kommunikation };

/*MenuChild DatumUhrzeit[] = {
	{"Datum", NULL, NULL},
	{"Uhrzeit", NULL, NULL},
};

Menu MenuEinstellungenUhrzeit = { NR_ENTRIES( DatumUhrzeit ), DatumUhrzeit };*/

MenuChild System[] = {
	{"Kommunikation", &MenuEinstellungenKommunikation, NULL},
	{"Datum / Uhrzeit", NULL, DatumUhrzeitStellen}, 
};


Menu MenuSystem = { NR_ENTRIES( System ), System};

MenuChild Beleuchtung[] = {
	{"Type", NULL, BeleuchtungTypeEinstellen},
};

Menu MenuBeleuchtung = { NR_ENTRIES( Beleuchtung ), Beleuchtung };

MenuChild Temperaturen[] = {
	{"Raumtemperatur", NULL, RaumtemperaturEinstellen},
	{"Absenktemp.", NULL, AbsenktemperaturEinstellen},
};

Menu MenuTemperaturen = { NR_ENTRIES( Temperaturen ), Temperaturen};

MenuChild Temperatur[] = {
	{"Temperaturen", &MenuTemperaturen, NULL},
	{"Zeitsteuerung", NULL, NULL},
	{"Type", NULL, TemperaturTypeEinstellen},
	{"kalibrieren", NULL, NULL},
};

Menu MenuTemperatur = { NR_ENTRIES( Temperatur ), Temperatur};

MenuChild Rolladen[] = {
	{"Zeitsteuerung", NULL, NULL},
	{"kalibrieren", NULL, NULL},
};

Menu MenuRolladen = { NR_ENTRIES( Rolladen ), Rolladen};

MenuChild Fernsteuerung[] = {
	{"lernen", NULL, NULL},
};

Menu MenuFernsteuerung = { NR_ENTRIES( Fernsteuerung ), Fernsteuerung};

MenuChild HauptMenu[] = {
	{"System", &MenuSystem, NULL},
	{"Beleuchtung", &MenuBeleuchtung, NULL},
	{"Temperatur", &MenuTemperatur, NULL},
	{"Rolladen", &MenuRolladen, NULL},
	{"Fernsteuerung", &MenuFernsteuerung, NULL},
};

Menu MenuMain = { NR_ENTRIES( HauptMenu ), HauptMenu};

//Tasten
uint8_t tAuf = 0b01111111;
uint8_t tAb = 0b10111111;
uint8_t tRechts = 0b11011111;
uint8_t tLinks = 0b11101111;
uint8_t tMenu = 0b00111111; // auf + ab

//Menu
int8_t ImMenu = 0;
int8_t myMenu = 0;
int8_t ImUMenu = 0;
int8_t myUMenu = 0;
int8_t myUUMenu = 0;

int8_t KommunikationAktiviert;
int8_t KommunikationGeschwindigkeit;
uint8_t KommunikationAdresse;
int8_t BeleuchtungType;
int8_t Raumtemperatur;
int8_t Absenktemperatur;
int8_t TemperaturType;

void Init(void) //Komponenten initialisieren
{
	//Taster einrichten
	DDRB = 0x00;
	PORTB = 0xff;
	
	//Ausgnge einrichten
	DDRE = 0xff;
	PORTE = 0b11110000;
	
	//Display initialisieren
	lcd_init();
	
	//RTC initialisieren
	rtcInit();
	
	KommunikationAktiviert = eeprom_read_byte(1);
		if (KommunikationAktiviert < 0){KommunikationAktiviert = 1;}
		
	KommunikationGeschwindigkeit = eeprom_read_byte(2);
		if (KommunikationGeschwindigkeit < 0){KommunikationGeschwindigkeit = 3;}
		
	KommunikationAdresse = eeprom_read_byte(3);
		if (KommunikationAdresse < 0){KommunikationAdresse = 1;}
		
	BeleuchtungType = eeprom_read_byte(4);
		if (BeleuchtungType < 0){BeleuchtungType = 1;} //0 = Dimmer, 1 = Schalter
	
	Raumtemperatur = eeprom_read_byte(5);
		if (Raumtemperatur < 0){Raumtemperatur = 22;}
	
	Absenktemperatur = eeprom_read_byte(6);
		if (Absenktemperatur < 0){Absenktemperatur = 15;}
		
	TemperaturType = eeprom_read_byte(7);
		if (TemperaturType < 0){TemperaturType = 0;} //0 = Thermostat, 1 = Regelung
	
}


uint8_t KeyPress(){

	if (PINB != 0xff) 
		{
			uint8_t keystate = PINB; //Tastenwert abfragen
		
			_delay_ms(50);
			_delay_ms(50);
			
			if (PINB == keystate)
			{
				return PINB;
			}
		}
	
	
	return 0;

}

void Navigate(void){

	if (PINB != 0xff) 
		{
			uint8_t keystate = PINB; //Tastenwert abfragen
		
			_delay_ms(50);
			_delay_ms(50);
			
			if (PINB == keystate)
			{
				
				lcd_clear();
				
				if (PINB == tMenu)
				{
											
					//Warten bis Tasten losgelassen
					while (PINB != 0xff){
					
					}
					
					if (ImMenu == 0) {
						ImMenu = 1;
					} else {
						ImMenu = 0;
						myMenu = 0;
						myUMenu = 0;
						ImUMenu = 0;
						myUUMenu = 0;
					}
				
									
				} else if (PINB == tAuf){
					
					if (ImMenu == 1){
						if (ImUMenu == 0) {
							myMenu -= 1;
					
							if (myMenu < 0) {
								myMenu = MenuMain.nrChilds -1;
							}
						} else if (ImUMenu == 1) {
							myUMenu -= 1;
							
							if (myUMenu < 0) {
								myUMenu = MenuMain.childs[myMenu].subMenu->nrChilds -1;
							}
						} else {
							myUUMenu -= 1;
							
							if (myUUMenu < 0) {
								myUUMenu = MenuMain.childs[myMenu].subMenu->childs[myUMenu].subMenu->nrChilds -1;
							}
						}
					}
					
									
				} else if (PINB == tAb)	{
				
					if (ImMenu == 1){
						if (ImUMenu == 0) {
							myMenu += 1;
					
							if (myMenu > MenuMain.nrChilds -1) {
								myMenu = 0;
							}
						} else if (ImUMenu == 1) {
							myUMenu += 1;
							
							if (myUMenu > MenuMain.childs[myMenu].subMenu->nrChilds -1) {
								myUMenu = 0;
							}
						} else {
							myUUMenu += 1;
							
							if (myUUMenu > MenuMain.childs[myMenu].subMenu->childs[myUMenu].subMenu->nrChilds -1) {
								myUUMenu = 0;
							}
						}
					}
				
														
				} else if (PINB == tLinks){
					//zurck
					if (ImMenu == 1){
						if (ImUMenu > 0){
							ImUMenu -= 1;
							myUUMenu = 0;
							myUMenu = 0;
						} else {
							ImMenu = 0;
							myMenu = 0;
							myUMenu = 0;
							ImUMenu = 0;
							myUUMenu = 0;
						}
					}
					
								
				} else if (PINB == tRechts)	{
					//Besttigen
					if (ImMenu == 1){
						
						if (ImUMenu == 0){
							MenuChild myM = MenuMain.childs[myMenu];
						
							if (myM.myFunction != NULL){
								myM.myFunction();
							} else {
								ImUMenu += 1;
							}
						} else if (ImUMenu == 1){
							MenuChild myM = MenuMain.childs[myMenu].subMenu->childs[myUMenu];
							
							if (myM.myFunction != NULL){
								myM.myFunction();
							} else {
								ImUMenu += 1;
							}
						} else if (ImUMenu == 2){
							MenuChild myM = MenuMain.childs[myMenu].subMenu->childs[myUMenu].subMenu->childs[myUUMenu];
							
							if (myM.myFunction != NULL){
								myM.myFunction();
							}
						}
						
						
					}
					
				}
				
			}	
	}
	
	if (ImMenu == 0) {
		lcd_setcursor(0,1);
		lcd_string("myHome.Ctrl");
		
		lcd_setcursor(0,2);

		char Buffer[20];
		sprintf(Buffer, "%.2d:%.2d:%.2d.%.3d", 			 rtcTime.hour,
														 rtcTime.min,
														 rtcTime.sec,
														 rtcTime.subSec);
														 
		lcd_string(Buffer);
		
		/*itoa(rtcTime.day, Buffer, 10);
		//lcd_string(Buffer);
		//lcd_string(".");
		
		itoa(rtcTime.month, Buffer, 10);
		lcd_string(Buffer);
		lcd_string(".");
		
		//itoa(rtcTime.year, Buffer, 10);
		//lcd_string(Buffer);
		//lcd_string(".");
		
		lcd_setcursor(9,2);
		
		itoa(rtcTime.hour, Buffer, 10);
		lcd_string(Buffer);
		lcd_string(":");
		
		itoa(rtcTime.min, Buffer, 10);
		lcd_string(Buffer);
		lcd_string(":");
		
		itoa(rtcTime.sec, Buffer, 10);
		lcd_string(Buffer);*/
		
	} else {
		
		if (ImUMenu < 2) {
			MenuChild myM = MenuMain.childs[myMenu];
		
			lcd_setcursor(0,1);
			lcd_string(myM.text);
		
			if (ImUMenu == 1) {
				lcd_setcursor(0,2);
				lcd_string(" ");
				MenuChild myUM = myM.subMenu->childs[myUMenu];
			
				lcd_string(myUM.text);
			}
		} else {
			MenuChild myM = MenuMain.childs[myMenu].subMenu->childs[myUMenu];
			MenuChild myUM = myM.subMenu->childs[myUUMenu];
			
			lcd_setcursor(0,1);
			lcd_string(myM.text);
			lcd_setcursor(0,2);
			lcd_string(" ");
			lcd_string(myUM.text);
		}
	}
	
}

void DatumUhrzeitStellen(void){
	lcd_clear();
	lcd_setcursor(0,1);
	lcd_string("Datum & Zeit");
			
	date_t Status;
	
	Status.day = rtcTime.day;
	Status.month = rtcTime.month;
	Status.year = rtcTime.year;
	Status.hour = rtcTime.hour;
	Status.min = rtcTime.min;
	Status.sec = rtcTime.sec;
			
	int8_t CursorPos = 1;
	
	//lcd_command(LCD_CURSOR_ON);
	//lcd_command(LCD_BLINKING_ON);
	
	while(1){
		
		uint8_t Key = KeyPress();
		
		if (Key == tAuf){
		
			if (CursorPos == 1){
				Status.day += 1;
			} else if (CursorPos == 4){
				Status.month += 1;
			} else if (CursorPos == 9){
				Status.year += 1;
			} else if (CursorPos == 12){
				Status.hour += 1;
			} else if (CursorPos == 15){
				Status.min += 1;
			}
		
					
		} else if (Key == tAb){
		
			if (CursorPos == 1){
				Status.day -= 1;
			} else if (CursorPos == 4){
				Status.month -= 1;
			} else if (CursorPos == 9){
				Status.year -= 1;
			} else if (CursorPos == 12){
				Status.hour -= 1;
			} else if (CursorPos == 15){
				Status.min -= 1;
			}
		
		} else if (Key == tLinks){
			
			
			if (CursorPos == 1){
				lcd_clear();
				//lcd_command(LCD_CURSOR_OFF);
				//lcd_command(LCD_BLINKING_OFF);
				break;
			} else if (CursorPos == 4){
				CursorPos = 1;
			} else if (CursorPos == 9){
				CursorPos = 4;
			} else if (CursorPos == 12){
				CursorPos = 9;
			} else if (CursorPos == 15){
				CursorPos = 12;
			}
			
		} else if (Key == tRechts){
						
			if (CursorPos == 15){
				lcd_clear();
				//lcd_command(LCD_CURSOR_OFF);
				//lcd_command(LCD_BLINKING_OFF);
				
				rtcTime.min = Status.min;
				rtcTime.hour = Status.hour;
				rtcTime.day = Status.day;
				rtcTime.month = Status.month;
				rtcTime.year = Status.year;
				
				//rtcTimeInit = 0x00;
				//rtcInit();
				
				/*rtcDateAdd(rtcTime, RTC_MIN, Status.min);
				rtcDateAdd(rtcTime, RTC_HOUR, Status.hour);
				rtcDateAdd(rtcTime, RTC_DAY, Status.day);
				rtcDateAdd(rtcTime, RTC_MONTH, Status.month);
				rtcDateAdd(rtcTime, RTC_YEAR, Status.year);*/
				
				break;
			} else if (CursorPos == 1){
				CursorPos = 4;
			} else if (CursorPos == 4){
				CursorPos = 9;
			} else if (CursorPos == 9){
				CursorPos = 12;
			} else if (CursorPos == 12){
				CursorPos = 15;
			}
		}
	
		lcd_clear();
		lcd_setcursor(CursorPos,1);
		lcd_string("_");
	
		lcd_setcursor(0,2);

		char Buffer[20];
		sprintf(Buffer, "%.2d.%.2d.%.2d %.2d:%.2d.%.2d", Status.day+1,
														 Status.month+1,
														 Status.year,
														 Status.hour,
														 Status.min,
														 Status.sec);
														 
		lcd_string(Buffer);
	
		//lcd_setcursor(CursorPos,2);
	
	}
}

void TemperaturTypeEinstellen(void){
	lcd_clear();
	lcd_setcursor(0,1);
	lcd_string("Temperaturregel.");
	
	int8_t Status = TemperaturType;
	
	while(1){
		uint8_t Key = KeyPress();
		
		if ((Key == tAuf) || (Key == tAb)){
			if (Status == 0){
				Status = 1;
			} else if (Status == 1){
				Status = 0;
			}
			
		} else if (Key == tLinks) {
			lcd_clear();
			break;
		} else if (Key == tRechts) {
			TemperaturType = Status;
			eeprom_write_byte(7, Status);
			lcd_clear();
			break;
		}
		
		lcd_setcursor(0,2);
		
		if (Status == TemperaturType){
			lcd_string("*");
		} else {
			lcd_string(" ");
		}
		
		if (Status == 0){
			lcd_string("Thermostat");
		} else if (Status == 1) {
			lcd_string("Regelung");
		}
		
		lcd_string("  ");
	}
}

void AbsenktemperaturEinstellen(void){
	lcd_clear();
	lcd_setcursor(0,1);
	lcd_string("Abs.temp. einst.");
	
	int8_t Status = Absenktemperatur;
	
	while(1){
		uint8_t Key = KeyPress();
		
		if (Key == tAb) {
			Status -= 1;
			
			if (Status < 0) {Status = 0;}
		} else if (Key == tAuf) {
			Status += 1;
			
			if (Status > 50) {Status = 50;}
		} else if (Key == tLinks) {
			lcd_clear();
			break;
		} else if (Key == tRechts) {
			Absenktemperatur = Status;
			eeprom_write_byte(6, Status);
			lcd_clear();
			break;
		}
		
		lcd_setcursor(0,2);
		
		if (Status == Absenktemperatur){
			lcd_string("*");
		} else {
			lcd_string(" ");
		}
		
		char Buffer[20];
		itoa(Status, Buffer, 10);
		lcd_string(Buffer);
		lcd_string(" C");
		lcd_string("  ");
	}
}


void RaumtemperaturEinstellen(void){
	lcd_clear();
	lcd_setcursor(0,1);
	lcd_string("Raumtemp. einst.");
	
	int8_t Status = Raumtemperatur;
	
	while(1){
		uint8_t Key = KeyPress();
		
		if (Key == tAb) {
			Status -= 1;
			
			if (Status < 0) {Status = 0;}
		} else if (Key == tAuf) {
			Status += 1;
			
			if (Status > 50) {Status = 50;}
		} else if (Key == tLinks) {
			lcd_clear();
			break;
		} else if (Key == tRechts) {
			Raumtemperatur = Status;
			eeprom_write_byte(5, Status);
			lcd_clear();
			break;
		}
		
		lcd_setcursor(0,2);
		
		if (Status == Raumtemperatur){
			lcd_string("*");
		} else {
			lcd_string(" ");
		}
		
		char Buffer[20];
		itoa(Status, Buffer, 10);
		lcd_string(Buffer);
		lcd_string(" C");
		lcd_string("  ");
	}
}

void BeleuchtungTypeEinstellen(void){
	lcd_clear();
	lcd_setcursor(0,1);
	lcd_string("Beleuchtung Type");
	
	int8_t Status = BeleuchtungType;
	
	while(1){
		uint8_t Key = KeyPress();
		
		if ((Key == tAuf) || (Key == tAb)){
			if (Status == 0){
				Status = 1;
			} else {
				Status = 0;
			}
		} else if (Key == tLinks){
			lcd_clear();
			break;
		} else if (Key == tRechts){
			BeleuchtungType = Status;
			eeprom_write_byte(4, Status);
			lcd_clear();
			break;
		}
		
		lcd_setcursor(0,2);
		
		if (Status == BeleuchtungType){
			lcd_string("*");
		} else {
			lcd_string(" ");
		}
		
		if (Status == 0){
			lcd_string("Dimmer"  );
		} else if (Status == 1) {
			lcd_string("Schalter");
		}
	}
}

void KommunikationAdresseEinstellen(void){
	lcd_clear();
	lcd_setcursor(0,1);
	lcd_string("Kom. Adresse");
	
	uint8_t Status = KommunikationAdresse;
	
	while(1){
		uint8_t Key = KeyPress();
		
		if (Key == tAb){
			if (Status == 255){
				Status = 1;
			} else {
				Status += 1;
			}
		} else if (Key == tAuf){
			Status -= 1;
			
			if (Status < 1){Status = 255;}
		} else if (Key == tLinks){
			lcd_clear();
			break;
		} else if (Key == tRechts){
			KommunikationAdresse = Status;
			eeprom_write_byte(3, Status);
			lcd_clear();
			break;
		}
		
		lcd_setcursor(0,2);
		
		if (Status == KommunikationAdresse){
			lcd_string("*");
		} else {
			lcd_string(" ");
		}
		
		char Buffer[20];
		itoa(Status, Buffer, 10);
		lcd_string(Buffer);
		lcd_string("  ");
		
	}
}

void KommunikationGeschwindigkeitEinstellen(void){
	lcd_clear();
	lcd_setcursor(0,1);
	lcd_string("Kom. Geschw.");
	
	int8_t Status = KommunikationGeschwindigkeit;
	
	while(1){
		uint8_t Key = KeyPress();
		
		if (Key == tAb){
			Status += 1;
			
			if (Status > 7){Status = 0;}
		} else if (Key == tAuf){
			Status -= 1;
			if (Status < 0){Status = 7;}
		} else if (Key == tLinks){
			lcd_clear();
			break;
		} else if (Key == tRechts){
			KommunikationGeschwindigkeit = Status;
			eeprom_write_byte(2, Status);
			lcd_clear();
			break;
		}
		
		lcd_setcursor(0,2);
		
		if (Status == KommunikationGeschwindigkeit){
			lcd_string("*");
		} else {
			lcd_string(" ");
		}
		
		switch(Status){
			case 0: lcd_string("1200  ");
					break;
			case 1: lcd_string("2400  ");
					break;
			case 2: lcd_string("4800  ");
					break;
			case 3: lcd_string("9600  ");
					break;
			case 4: lcd_string("19200 ");
					break;
			case 5: lcd_string("38400 ");
					break;
			case 6: lcd_string("57600 ");
					break;
			case 7: lcd_string("115200");
					break;
		}
	}
}

void KommunikationAktivieren(void){
	lcd_clear();
	lcd_setcursor(0,1);
	lcd_string("Kom. aktiv");
	
	int8_t Status = KommunikationAktiviert;
	
	while(1){
		uint8_t Key = KeyPress();
	
		if ((Key == tAuf) || (Key == tAb)){
			if (Status == 1){
				Status = 0;
			} else {
				Status = 1;
			}
					
		} else if (Key == tLinks) {
			lcd_clear();
			break;
		} else if (Key == tRechts) {
			KommunikationAktiviert = Status;
			eeprom_write_byte(1, Status);
			lcd_clear();
			break;
		}
		
		lcd_setcursor(0,2);
			
		if (Status == KommunikationAktiviert){
			lcd_string("*");
		} else {
			lcd_string(" ");
		}
			
		if (Status != 0) {
			lcd_string("aktiviert  ");
		} else {
			lcd_string("deaktiviert");
		}
	}
}

int main(void){
		
	Init();
	
	lcd_setcursor(0,1);
	lcd_string("myHome.Control  ");
	lcd_setcursor(0,2);
	lcd_string("Initialisieren  ");

	//Connect();
	
	_delay_ms(50);
	_delay_ms(50);
	_delay_ms(50);
	_delay_ms(50);
	
	lcd_clear();
	
	while(1)
	{
		Navigate();
	}
	
	return 0;
}