/*
 * Copyright:      Fabian Maximilian Thiele  mailto:me@apetech.de
 * Author:         Fabian Maximilian Thiele
 * Remarks:        this Copyright must be included
 * known Problems: none
 * Version:        1.0
 * Description:    Real-Time-Clock mit Wochentagsberechnung.
 * 
 */

#ifndef RTC_H
#define RTC_H


//
// Defines
//

// auskommentieren wenn der Scheduler nicht benutzt wird
//#define USE_SCHEDULER

// AVR-Taktfrequenz
//#define F_OSC				14745600UL
// RTC Interrupts pro Sekunde (fr 100 -> kleinste Zeiteinheit = 1/100s)
#define RTC_IRQS_SEC		100
#define RTC_TCOMP_VAL		144
#define RTC_TPS_VAL			(1<<CS00) | (1<<CS02) //1024

// Zeit-Intervalle
#define RTC_SUB_SEC			1
#define RTC_SEC				2
#define RTC_MIN				3
#define RTC_HOUR			4
#define RTC_DAY				5
#define RTC_MONTH			6
#define RTC_YEAR			7

// Wochentage
#define RTC_MONDAY			0
#define RTC_TUESDAY			1
#define RTC_WEDNESDAY		2
#define RTC_THURSDAY		3
#define RTC_FRIDAY			4
#define RTC_SATURDAY		5
#define RTC_SUNDAY			6


//
// Typedefs
//

/*
 * Datentyp fr Datum und Uhrzeit.
 * Zu beachten ist, dass der erste Tag im Monat den Wert 0 hat und die Monate
 * ebenfalls bei 0 anfangen zu zhlen.
 */
typedef struct {
	uint16_t subSec;		// 0 .. RTC_IRQS_SEC
	uint8_t sec;			// 0 .. 59
	uint8_t min;			// 0 .. 59
	uint8_t hour;			// 0 .. 23
	uint8_t day;			// 0 .. rtcDaysOfMonth(month, year)-1
	uint8_t month;			// 0 .. 11
	uint16_t year;			// 0 .. 65535 :)
} date_t;


//
// Funktions Prototypen
//

// Timer initialisieren
void rtcInit(void);
// Vergleicht ein beliebiges Datum mit dem Systemdatum. Gibt -1 zurck, wenn
//  frher, 0 wenn gleich und 1 wenn spter
int8_t rtcCompToSystime(date_t* date);
// Fgt einem Datum ein bestimmtes Zeitinterval hinzu
void rtcDateAdd(date_t* date, uint8_t incType, uint32_t value);
// Gibt den Wochentag eines beliebigen Datums zurck;
uint8_t rtcDayOfWeek(date_t* date);
// Gibt die Anzahl der Tage eines bestimmten Monats zurck
uint8_t rtcDaysOfMonth(uint8_t month, uint16_t year);
// Gibt 1 zurck, wenn y ein Schaltjahr ist, sonst 0
uint8_t rtcLeapYear(uint16_t y);
// Wartet ms Millisekunden
void rtcWaitMS(uint16_t ms);


//
// Globale Variablen
//

// RTC Zeit
extern volatile date_t rtcTime;

#endif
